'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var draftJs = require('draft-js');
var Editor = require('draft-js-plugins-editor');
var draftjsConductor = require('draftjs-conductor');
var decorateComponentWithProps = require('decorate-component-with-props');
var isSoftNewlineEvent = require('draft-js/lib/isSoftNewlineEvent');
var draftjsFilters = require('draftjs-filters');
var Tippy = require('@tippyjs/react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var Editor__default = /*#__PURE__*/_interopDefaultLegacy(Editor);
var decorateComponentWithProps__default = /*#__PURE__*/_interopDefaultLegacy(decorateComponentWithProps);
var isSoftNewlineEvent__default = /*#__PURE__*/_interopDefaultLegacy(isSoftNewlineEvent);
var Tippy__default = /*#__PURE__*/_interopDefaultLegacy(Tippy);

const BLOCK_TYPE = {
  // This is used to represent a normal text block (paragraph).
  UNSTYLED: "unstyled",
  HEADER_ONE: "header-one",
  HEADER_TWO: "header-two",
  HEADER_THREE: "header-three",
  HEADER_FOUR: "header-four",
  HEADER_FIVE: "header-five",
  HEADER_SIX: "header-six",
  UNORDERED_LIST_ITEM: "unordered-list-item",
  ORDERED_LIST_ITEM: "ordered-list-item",
  BLOCKQUOTE: "blockquote",
  CODE: "code-block",
  // This represents a "custom" block, not for rich text, with arbitrary content.
  ATOMIC: "atomic"
};
const ENTITY_TYPE = {
  LINK: "LINK",
  IMAGE: "IMAGE",
  HORIZONTAL_RULE: "HORIZONTAL_RULE"
}; // See https://github.com/facebook/draft-js/blob/master/src/model/immutable/DefaultDraftInlineStyle.js

const INLINE_STYLE = {
  BOLD: "BOLD",
  ITALIC: "ITALIC",
  CODE: "CODE",
  UNDERLINE: "UNDERLINE",
  STRIKETHROUGH: "STRIKETHROUGH",
  MARK: "MARK",
  QUOTATION: "QUOTATION",
  SMALL: "SMALL",
  SAMPLE: "SAMPLE",
  INSERT: "INSERT",
  DELETE: "DELETE",
  KEYBOARD: "KEYBOARD",
  SUPERSCRIPT: "SUPERSCRIPT",
  SUBSCRIPT: "SUBSCRIPT"
};
const BLOCK_TYPES = Object.values(BLOCK_TYPE);
const ENTITY_TYPES = Object.values(ENTITY_TYPE);
const INLINE_STYLES = Object.values(INLINE_STYLE);
const KEY_COMMANDS = [...BLOCK_TYPES, ...ENTITY_TYPES, ...INLINE_STYLES, // Lowercase identifiers used by Draft.js
// See https://github.com/facebook/draft-js/blob/585af35c3a8c31fefb64bc884d4001faa96544d3/src/model/constants/DraftEditorCommand.js#L58-L61.
"bold", "italic", "underline", "code"];
const FONT_FAMILY_MONOSPACE = "Consolas, Menlo, Monaco, Lucida Console, Liberation Mono, DejaVu Sans Mono, Bitstream Vera Sans Mono, Courier New, monospace, sans-serif"; // See https://github.com/facebook/draft-js/blob/master/src/model/immutable/DefaultDraftInlineStyle.js

const CUSTOM_STYLE_MAP = {};
CUSTOM_STYLE_MAP[INLINE_STYLE.BOLD] = draftJs.DefaultDraftInlineStyle[INLINE_STYLE.BOLD];
CUSTOM_STYLE_MAP[INLINE_STYLE.ITALIC] = draftJs.DefaultDraftInlineStyle[INLINE_STYLE.ITALIC];
CUSTOM_STYLE_MAP[INLINE_STYLE.STRIKETHROUGH] = draftJs.DefaultDraftInlineStyle[INLINE_STYLE.STRIKETHROUGH];
CUSTOM_STYLE_MAP[INLINE_STYLE.UNDERLINE] = draftJs.DefaultDraftInlineStyle[INLINE_STYLE.UNDERLINE];
CUSTOM_STYLE_MAP[INLINE_STYLE.CODE] = {
  padding: "0.2em 0.3125em",
  margin: "0",
  fontSize: "85%",
  backgroundColor: "rgba(27, 31, 35, 0.05)",
  fontFamily: FONT_FAMILY_MONOSPACE,
  borderRadius: "3px"
};
CUSTOM_STYLE_MAP[INLINE_STYLE.MARK] = {
  backgroundColor: "yellow"
};
CUSTOM_STYLE_MAP[INLINE_STYLE.QUOTATION] = {
  fontStyle: "italic"
};
CUSTOM_STYLE_MAP[INLINE_STYLE.SMALL] = {
  fontSize: "smaller"
};
CUSTOM_STYLE_MAP[INLINE_STYLE.SAMPLE] = {
  fontFamily: FONT_FAMILY_MONOSPACE
};
CUSTOM_STYLE_MAP[INLINE_STYLE.INSERT] = {
  textDecoration: "underline"
};
CUSTOM_STYLE_MAP[INLINE_STYLE.DELETE] = {
  textDecoration: "line-through"
};
CUSTOM_STYLE_MAP[INLINE_STYLE.KEYBOARD] = {
  fontFamily: FONT_FAMILY_MONOSPACE,
  padding: "3px 5px",
  fontSize: "11px",
  lineHeight: "10px",
  color: "#444d56",
  verticalAlign: "middle",
  backgroundColor: "#fafbfc",
  border: "solid 1px #c6cbd1",
  borderBottomColor: "#959da5",
  borderRadius: "3px",
  boxShadow: "inset 0 -1px 0 #959da5"
};
CUSTOM_STYLE_MAP[INLINE_STYLE.SUPERSCRIPT] = {
  fontSize: "80%",
  verticalAlign: "super",
  lineHeight: "1"
};
CUSTOM_STYLE_MAP[INLINE_STYLE.SUBSCRIPT] = {
  fontSize: "80%",
  verticalAlign: "sub",
  lineHeight: "1"
};
const BR_TYPE = "BR";
const UNDO_TYPE = "undo";
const REDO_TYPE = "redo"; // Originally from https://github.com/facebook/draft-js/blob/master/src/component/utils/getDefaultKeyBinding.js.

const KEY_CODES = {
  K: 75,
  B: 66,
  U: 85,
  J: 74,
  I: 73,
  X: 88,
  "0": 48,
  "1": 49,
  "2": 50,
  "3": 51,
  "4": 52,
  "5": 53,
  "6": 54,
  "7": 55,
  "8": 56,
  ".": 190,
  ",": 188
};
const INPUT_BLOCK_MAP = {
  "* ": BLOCK_TYPE.UNORDERED_LIST_ITEM,
  "- ": BLOCK_TYPE.UNORDERED_LIST_ITEM,
  "1. ": BLOCK_TYPE.ORDERED_LIST_ITEM,
  "# ": BLOCK_TYPE.HEADER_ONE,
  "## ": BLOCK_TYPE.HEADER_TWO,
  "### ": BLOCK_TYPE.HEADER_THREE,
  "#### ": BLOCK_TYPE.HEADER_FOUR,
  "##### ": BLOCK_TYPE.HEADER_FIVE,
  "###### ": BLOCK_TYPE.HEADER_SIX,
  "> ": BLOCK_TYPE.BLOCKQUOTE,
  // It makes more sense not to require a space here.
  // This matches how Dropbox Paper operates.
  "```": BLOCK_TYPE.CODE
};
const INPUT_STYLE_MAP = [// Order matters, as shorter patterns are contained in the longer ones.
{
  pattern: "**",
  type: INLINE_STYLE.BOLD
}, {
  pattern: "__",
  type: INLINE_STYLE.BOLD
}, {
  pattern: "*",
  type: INLINE_STYLE.ITALIC
}, {
  pattern: "_",
  type: INLINE_STYLE.ITALIC
}, {
  pattern: "~~",
  type: INLINE_STYLE.STRIKETHROUGH
}, {
  pattern: "~",
  type: INLINE_STYLE.STRIKETHROUGH
}, {
  pattern: "`",
  type: INLINE_STYLE.CODE
}].map(_ref => {
  let {
    pattern,
    type
  } = _ref;
  const pat = pattern.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
  const char = pattern[0].replace(/[.*+?^${}()|[\]\\]/g, "\\$&"); // https://regexper.com/#%28%5Cs%7C%5E%29__%28%5B%5E%5Cs_%5D%7B1%2C2%7D%7C%5B%5E%5Cs_%5D.%2B%5B%5E%5Cs_%5D%29__%24
  // This is stored as an escaped string instead of a RegExp object because they are stateful.
  // This regex encapsulates a few rules:
  // - The pattern must be preceded by whitespace, or be at the start of the input.
  // - The pattern must end the input.
  // - In-between the start and end patterns, there can't be only whitespace or characters from the pattern.
  // - There has to be at least 1 char that's not whitespace or the pattern’s char.

  const regex = `(\\s|^)${pat}([^\\s${char}]{1,2}|[^\\s${char}].+[^\\s${char}])${pat}$`;
  return {
    pattern,
    type,
    regex
  };
});
const INPUT_ENTITY_MAP = {};
INPUT_ENTITY_MAP[ENTITY_TYPE.HORIZONTAL_RULE] = "---";
const LABELS = {};
LABELS[BLOCK_TYPE.UNSTYLED] = "P";
LABELS[BLOCK_TYPE.HEADER_ONE] = "H1";
LABELS[BLOCK_TYPE.HEADER_TWO] = "H2";
LABELS[BLOCK_TYPE.HEADER_THREE] = "H3";
LABELS[BLOCK_TYPE.HEADER_FOUR] = "H4";
LABELS[BLOCK_TYPE.HEADER_FIVE] = "H5";
LABELS[BLOCK_TYPE.HEADER_SIX] = "H6";
LABELS[BLOCK_TYPE.UNORDERED_LIST_ITEM] = "UL";
LABELS[BLOCK_TYPE.ORDERED_LIST_ITEM] = "OL";
LABELS[BLOCK_TYPE.CODE] = "{ }";
LABELS[BLOCK_TYPE.BLOCKQUOTE] = "❝";
LABELS[INLINE_STYLE.BOLD] = "B";
LABELS[INLINE_STYLE.ITALIC] = "𝘐";
LABELS[INLINE_STYLE.CODE] = "{ }";
LABELS[INLINE_STYLE.UNDERLINE] = "U";
LABELS[INLINE_STYLE.STRIKETHROUGH] = "S";
LABELS[INLINE_STYLE.MARK] = "☆";
LABELS[INLINE_STYLE.QUOTATION] = "❛";
LABELS[INLINE_STYLE.SMALL] = "𝖲𝗆a𝗅𝗅";
LABELS[INLINE_STYLE.SAMPLE] = "𝙳𝚊𝚝𝚊";
LABELS[INLINE_STYLE.INSERT] = "Ins";
LABELS[INLINE_STYLE.DELETE] = "Del";
LABELS[INLINE_STYLE.SUPERSCRIPT] = "Sup";
LABELS[INLINE_STYLE.SUBSCRIPT] = "Sub";
LABELS[INLINE_STYLE.KEYBOARD] = "⌘";
LABELS[ENTITY_TYPE.LINK] = "🔗";
LABELS[ENTITY_TYPE.IMAGE] = "🖼";
LABELS[ENTITY_TYPE.HORIZONTAL_RULE] = "―";
LABELS[BR_TYPE] = "↵";
LABELS[UNDO_TYPE] = "↺";
LABELS[REDO_TYPE] = "↻";
const DESCRIPTIONS = {};
DESCRIPTIONS[BLOCK_TYPE.UNSTYLED] = "Paragraph";
DESCRIPTIONS[BLOCK_TYPE.HEADER_ONE] = "Heading 1";
DESCRIPTIONS[BLOCK_TYPE.HEADER_TWO] = "Heading 2";
DESCRIPTIONS[BLOCK_TYPE.HEADER_THREE] = "Heading 3";
DESCRIPTIONS[BLOCK_TYPE.HEADER_FOUR] = "Heading 4";
DESCRIPTIONS[BLOCK_TYPE.HEADER_FIVE] = "Heading 5";
DESCRIPTIONS[BLOCK_TYPE.HEADER_SIX] = "Heading 6";
DESCRIPTIONS[BLOCK_TYPE.UNORDERED_LIST_ITEM] = "Bulleted list";
DESCRIPTIONS[BLOCK_TYPE.ORDERED_LIST_ITEM] = "Numbered list";
DESCRIPTIONS[BLOCK_TYPE.BLOCKQUOTE] = "Blockquote";
DESCRIPTIONS[BLOCK_TYPE.CODE] = "Code block";
DESCRIPTIONS[INLINE_STYLE.BOLD] = "Bold";
DESCRIPTIONS[INLINE_STYLE.ITALIC] = "Italic";
DESCRIPTIONS[INLINE_STYLE.CODE] = "Code";
DESCRIPTIONS[INLINE_STYLE.UNDERLINE] = "Underline";
DESCRIPTIONS[INLINE_STYLE.STRIKETHROUGH] = "Strikethrough";
DESCRIPTIONS[INLINE_STYLE.MARK] = "Highlight";
DESCRIPTIONS[INLINE_STYLE.QUOTATION] = "Inline quotation";
DESCRIPTIONS[INLINE_STYLE.SMALL] = "Small";
DESCRIPTIONS[INLINE_STYLE.SAMPLE] = "Program output";
DESCRIPTIONS[INLINE_STYLE.INSERT] = "Inserted";
DESCRIPTIONS[INLINE_STYLE.DELETE] = "Deleted";
DESCRIPTIONS[INLINE_STYLE.KEYBOARD] = "Shortcut key";
DESCRIPTIONS[INLINE_STYLE.SUPERSCRIPT] = "Superscript";
DESCRIPTIONS[INLINE_STYLE.SUBSCRIPT] = "Subscript";
DESCRIPTIONS[ENTITY_TYPE.LINK] = "Link";
DESCRIPTIONS[ENTITY_TYPE.IMAGE] = "Image";
DESCRIPTIONS[ENTITY_TYPE.HORIZONTAL_RULE] = "Horizontal line";
DESCRIPTIONS[BR_TYPE] = "Line break";
DESCRIPTIONS[UNDO_TYPE] = "Undo";
DESCRIPTIONS[REDO_TYPE] = "Redo";
const KEYBOARD_SHORTCUTS = {};
KEYBOARD_SHORTCUTS[BLOCK_TYPE.UNSTYLED] = "⌫";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.HEADER_ONE] = "#";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.HEADER_TWO] = "##";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.HEADER_THREE] = "###";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.HEADER_FOUR] = "####";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.HEADER_FIVE] = "#####";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.HEADER_SIX] = "######";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.UNORDERED_LIST_ITEM] = "-";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.ORDERED_LIST_ITEM] = "1.";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.BLOCKQUOTE] = ">";
KEYBOARD_SHORTCUTS[BLOCK_TYPE.CODE] = "```";
KEYBOARD_SHORTCUTS[INLINE_STYLE.BOLD] = {
  other: "Ctrl + B",
  macOS: "⌘ + B"
};
KEYBOARD_SHORTCUTS[INLINE_STYLE.ITALIC] = {
  other: "Ctrl + I",
  macOS: "⌘ + I"
};
KEYBOARD_SHORTCUTS[INLINE_STYLE.UNDERLINE] = {
  other: "Ctrl + U",
  macOS: "⌘ + U"
};
KEYBOARD_SHORTCUTS[INLINE_STYLE.STRIKETHROUGH] = {
  other: "Ctrl + ⇧ + X",
  macOS: "⌘ + ⇧ + X"
};
KEYBOARD_SHORTCUTS[INLINE_STYLE.SUPERSCRIPT] = {
  other: "Ctrl + .",
  macOS: "⌘ + ."
};
KEYBOARD_SHORTCUTS[INLINE_STYLE.SUBSCRIPT] = {
  other: "Ctrl + ,",
  macOS: "⌘ + ,"
};
KEYBOARD_SHORTCUTS[ENTITY_TYPE.LINK] = {
  other: "Ctrl + K",
  macOS: "⌘ + K"
};
KEYBOARD_SHORTCUTS[BR_TYPE] = "⇧ + ↵";
KEYBOARD_SHORTCUTS[ENTITY_TYPE.HORIZONTAL_RULE] = "- - -";
KEYBOARD_SHORTCUTS[UNDO_TYPE] = {
  other: "Ctrl + Z",
  macOS: "⌘ + Z"
};
KEYBOARD_SHORTCUTS[REDO_TYPE] = {
  other: "Ctrl + ⇧ + Z",
  macOS: "⌘ + ⇧ + Z"
};
const HANDLED = "handled";
const NOT_HANDLED = "not-handled";

/**
 * Inspired by draftjs-utils, with our custom functions.
 *
 * DraftUtils functions are utility helpers useful in isolation, specific to the Draft.js API,
 * without ties to Draftail's specific behavior or other APIs.
 */

var DraftUtils = {
  /**
   * Returns the first selected block.
   */
  getSelectedBlock(editorState) {
    const selection = editorState.getSelection();
    const content = editorState.getCurrentContent();
    return content.getBlockMap().get(selection.getStartKey());
  },

  /**
   * Returns the entity applicable to whole of current selection.
   * An entity can not span multiple blocks.
   * https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/inline.js#L75
   */
  getSelectionEntity(editorState) {
    let entity;
    const selection = editorState.getSelection();
    let start = selection.getStartOffset();
    let end = selection.getEndOffset();

    if (start === end && start === 0) {
      end = 1;
    } else if (start === end) {
      start -= 1;
    }

    const block = this.getSelectedBlock(editorState);

    for (let i = start; i < end; i += 1) {
      const currentEntity = block.getEntityAt(i);

      if (!currentEntity) {
        entity = undefined;
        break;
      }

      if (i === start) {
        entity = currentEntity;
      } else if (entity !== currentEntity) {
        entity = undefined;
        break;
      }
    }

    return entity;
  },

  /**
   * Creates a selection on a given entity in the currently selected block.
   * Returns the current selection if no entity key is provided, or if the entity could not be found.
   */
  getEntitySelection(editorState, entityKey) {
    const selection = editorState.getSelection();

    if (!entityKey) {
      return selection;
    }

    const block = this.getSelectedBlock(editorState);
    let entityRange; // https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/inline.js#L111

    block.findEntityRanges(value => value.get("entity") === entityKey, (start, end) => {
      entityRange = {
        start,
        end
      };
    });

    if (!entityRange) {
      return selection;
    }

    return selection.merge({
      anchorOffset: selection.isBackward ? entityRange.end : entityRange.start,
      focusOffset: selection.isBackward ? entityRange.start : entityRange.end
    });
  },

  /**
   * Updates a given atomic block's entity, merging new data with the old one.
   */
  updateBlockEntity(editorState, block, data) {
    const content = editorState.getCurrentContent();
    let nextContent = content.mergeEntityData(block.getEntityAt(0), data); // To remove in Draft.js 0.11.
    // This is necessary because entity data is still using a mutable, global store.

    nextContent = draftJs.Modifier.mergeBlockData(nextContent, new draftJs.SelectionState({
      anchorKey: block.getKey(),
      anchorOffset: 0,
      focusKey: block.getKey(),
      focusOffset: block.getLength()
    }), {});
    return draftJs.EditorState.push(editorState, nextContent, "apply-entity");
  },

  /**
   * Inserts a horizontal rule in the place of the current selection.
   * Returns updated EditorState.
   * Inspired by DraftUtils.addLineBreakRemovingSelection.
   */
  addHorizontalRuleRemovingSelection(editorState) {
    const contentState = editorState.getCurrentContent();
    const contentStateWithEntity = contentState.createEntity( // Draft.js Flow typing issue.
    // See https://github.com/facebook/draft-js/issues/868.
    // $FlowFixMe
    ENTITY_TYPE.HORIZONTAL_RULE, "IMMUTABLE", {});
    const entityKey = contentStateWithEntity.getLastCreatedEntityKey();
    return draftJs.AtomicBlockUtils.insertAtomicBlock(editorState, entityKey, " ");
  },

  /**
   * Changes a block type to be `newType`, setting its new text.
   * Also removes the required characters from the characterList,
   * and resets block data.
   */
  resetBlockWithType(editorState, newType, newText) {
    let newData = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
    const contentState = editorState.getCurrentContent();
    const selectionState = editorState.getSelection();
    const key = selectionState.getStartKey();
    const blockMap = contentState.getBlockMap();
    const block = blockMap.get(key); // Maintain persistence in the list while removing chars from the start.
    // https://github.com/facebook/draft-js/blob/788595984da7c1e00d1071ea82b063ff87140be4/src/model/transaction/removeRangeFromContentState.js#L333

    let chars = block.getCharacterList();
    let startOffset = 0;
    const sliceOffset = block.getText().length - newText.length;

    while (startOffset < sliceOffset) {
      chars = chars.shift();
      startOffset += 1;
    }

    const newBlock = block.merge({
      type: newType,
      text: newText,
      characterList: chars,
      data: newData
    });
    const newContentState = contentState.merge({
      blockMap: blockMap.set(key, newBlock)
    });
    const newSelectionState = selectionState.merge({
      anchorOffset: 0,
      focusOffset: 0
    });
    return draftJs.EditorState.acceptSelection(draftJs.EditorState.set(editorState, {
      currentContent: newContentState
    }), newSelectionState);
  },

  /**
   * Applies an inline style on a given range, based on a Markdown shortcut,
   * removing the Markdown markers.
   * Supports adding styles on existing styles, and entities.
   */
  applyMarkdownStyle(editorState, range, char) {
    const selection = editorState.getSelection();
    let content = editorState.getCurrentContent();
    const marked = selection.merge({
      anchorOffset: range.start,
      focusOffset: range.end
    });
    const endMarker = selection.merge({
      anchorOffset: range.end - range.pattern.length,
      focusOffset: range.end
    });
    const startMarker = selection.merge({
      anchorOffset: range.start,
      focusOffset: range.start + range.pattern.length
    }); // Remove the markers separately to preserve existing styles and entities on the marked text.

    content = draftJs.Modifier.applyInlineStyle(content, marked, range.type);
    content = draftJs.Modifier.removeRange(content, endMarker, "forward");
    content = draftJs.Modifier.removeRange(content, startMarker, "forward");
    const offset = selection.getFocusOffset() - range.pattern.length * 2;
    const endSelection = selection.merge({
      anchorOffset: offset,
      focusOffset: offset
    });
    content = content.merge({
      selectionAfter: endSelection
    });
    content = draftJs.Modifier.insertText(content, endSelection, char);
    return draftJs.EditorState.push(editorState, content, "change-inline-style");
  },

  /**
   * Removes the block at the given key.
   */
  removeBlock(editorState, key) {
    const content = editorState.getCurrentContent();
    const blockMap = content.getBlockMap().remove(key);
    return draftJs.EditorState.set(editorState, {
      currentContent: content.merge({
        blockMap
      })
    });
  },

  /**
   * Removes a block-level entity, turning the block into an empty paragraph,
   * and placing the selection on it.
   */
  removeBlockEntity(editorState, entityKey, blockKey) {
    let newState = editorState;
    const content = editorState.getCurrentContent();
    const blockMap = content.getBlockMap();
    const block = blockMap.get(blockKey);
    const newBlock = block.merge({
      type: BLOCK_TYPE.UNSTYLED,
      text: "",
      // No text = no character list
      characterList: block.getCharacterList().slice(0, 0),
      data: {}
    });
    const newSelection = new draftJs.SelectionState({
      anchorKey: blockKey,
      focusKey: blockKey,
      anchorOffset: 0,
      focusOffset: 0
    });
    const newContent = content.merge({
      blockMap: blockMap.set(blockKey, newBlock)
    });
    newState = draftJs.EditorState.push(newState, newContent, "change-block-type");
    newState = draftJs.EditorState.forceSelection(newState, newSelection);
    return newState;
  },

  /**
   * Handles pressing delete within an atomic block. This can happen when selection is placed on an image.
   * Ideally this should be handled by the built-in RichUtils, but it's not.
   * See https://github.com/wagtail/wagtail/issues/4370.
   */
  handleDeleteAtomic(editorState) {
    const selection = editorState.getSelection();
    const content = editorState.getCurrentContent();
    const key = selection.getAnchorKey();
    const offset = selection.getAnchorOffset();
    const block = content.getBlockForKey(key); // Problematic selection. Pressing delete here would remove the entity, but not the block.

    if (selection.isCollapsed() && block.getType() === BLOCK_TYPE.ATOMIC && offset === 0) {
      return this.removeBlockEntity(editorState, block.getEntityAt(0), key);
    }

    return false;
  },

  /**
   * Get an entity decorator strategy based on the given entity type.
   * This strategy will find all entities of the given type.
   */
  getEntityTypeStrategy(entityType) {
    const strategy = (block, callback, contentState) => {
      block.findEntityRanges(character => {
        const entityKey = character.getEntity();
        return entityKey !== null && contentState.getEntity(entityKey).getType() === entityType;
      }, callback);
    };

    return strategy;
  },

  /**
   * Determines whether the editor should show its placeholder.
   * See https://draftjs.org/docs/api-reference-editor.html#placeholder
   * for details on why this is useful.
   */
  shouldHidePlaceholder(editorState) {
    const contentState = editorState.getCurrentContent();
    return contentState.hasText() || contentState.getBlockMap().first().getType() !== BLOCK_TYPE.UNSTYLED;
  },

  /**
   * Inserts new unstyled block.
   * Initially inspired from https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/block.js#L153,
   * but changed so that the split + block type reset amounts to
   * only one change in the undo stack.
   */
  insertNewUnstyledBlock(editorState) {
    const selection = editorState.getSelection();
    let newContent = draftJs.Modifier.splitBlock(editorState.getCurrentContent(), selection);
    const blockMap = newContent.getBlockMap();
    const blockKey = selection.getStartKey();
    const insertedBlockKey = newContent.getKeyAfter(blockKey);
    const newBlock = blockMap.get(insertedBlockKey).set("type", BLOCK_TYPE.UNSTYLED);
    newContent = newContent.merge({
      blockMap: blockMap.set(insertedBlockKey, newBlock)
    });
    return draftJs.EditorState.push(editorState, newContent, "split-block");
  },

  /**
   * Handles Shift + Enter keypress removing selection and inserting a line break.
   * https://github.com/jpuri/draftjs-utils/blob/112bbe449cc9156522fcf2b40f2910a071b795c2/js/block.js#L133
   */
  addLineBreak(editorState) {
    const content = editorState.getCurrentContent();
    const selection = editorState.getSelection();

    if (selection.isCollapsed()) {
      return draftJs.RichUtils.insertSoftNewline(editorState);
    }

    let newContent = draftJs.Modifier.removeRange(content, selection, "forward");
    const fragment = newContent.getSelectionAfter();
    const block = newContent.getBlockForKey(fragment.getStartKey());
    newContent = draftJs.Modifier.insertText(newContent, fragment, "\n", block.getInlineStyleAt(fragment.getStartOffset()), null);
    return draftJs.EditorState.push(editorState, newContent, "insert-fragment");
  },

  /**
   * Handles hard newlines.
   * https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/keyPress.js#L17
   */
  handleHardNewline(editorState) {
    const selection = editorState.getSelection();

    if (!selection.isCollapsed()) {
      return false;
    }

    const content = editorState.getCurrentContent();
    const blockKey = selection.getStartKey();
    const block = content.getBlockForKey(blockKey);
    const blockType = block.getType(); // Use a loose check to allow custom list item types to reuse the continuation behavior.

    const isListBlock = blockType.endsWith("-list-item");

    if (!isListBlock && block.getType() !== BLOCK_TYPE.UNSTYLED && block.getLength() === selection.getStartOffset()) {
      return this.insertNewUnstyledBlock(editorState);
    }

    if (isListBlock && block.getLength() === 0) {
      const depth = block.getDepth();

      if (depth === 0) {
        const nextContent = draftJs.RichUtils.tryToRemoveBlockStyle(editorState); // At the moment, tryToRemoveBlockStyle always returns for
        // collapsed selections at the start of a block. So in theory this corner case should never happen.

        return nextContent ? draftJs.EditorState.push(editorState, nextContent, "change-block-type") : false;
      }

      const blockMap = content.getBlockMap();
      const newBlock = block.set("depth", depth - 1);
      return draftJs.EditorState.push(editorState, content.merge({
        blockMap: blockMap.set(blockKey, newBlock)
      }), "adjust-depth");
    }

    return false;
  },

  /**
   * Handles three scenarios:
   * - Soft newlines.
   * - Hard newlines in the "defer breaking out of the block" case.
   * - Other hard newlines.
   * See https://github.com/springload/draftail/issues/104,
   * https://github.com/jpuri/draftjs-utils/issues/10.
   */
  handleNewLine(editorState, event) {
    // https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/keyPress.js#L64
    if (isSoftNewlineEvent__default['default'](event)) {
      return this.addLineBreak(editorState);
    }

    const content = editorState.getCurrentContent();
    const selection = editorState.getSelection();
    const key = selection.getStartKey();
    const offset = selection.getStartOffset();
    const block = content.getBlockForKey(key);
    const isDeferredBreakoutBlock = [BLOCK_TYPE.CODE].includes(block.getType());

    if (isDeferredBreakoutBlock) {
      const isEmpty = selection.isCollapsed() && offset === 0 && block.getLength() === 0;

      if (isEmpty) {
        return draftJs.EditorState.push(editorState, draftJs.Modifier.setBlockType(content, selection, BLOCK_TYPE.UNSTYLED), "change-block-type");
      }

      return false;
    }

    return this.handleHardNewline(editorState);
  }

};

const {
  hasCommandModifier,
  isOptionKeyCommand
} = draftJs.KeyBindingUtil;
const hasCmd = hasCommandModifier; // Hack relying on the internals of Draft.js.
// See https://github.com/facebook/draft-js/pull/869
// $FlowFixMe

const IS_MAC_OS = isOptionKeyCommand({
  altKey: "test"
}) === "test";
/**
 * Methods defining the behavior of the editor, depending on its configuration.
 */

var behavior = {
  /**
   * Configure block render map from block types list.
   */
  getBlockRenderMap(blockTypes) {
    let renderMap = draftJs.DefaultDraftBlockRenderMap; // Override default element for code block.
    // Fix https://github.com/facebook/draft-js/issues/406.

    if (blockTypes.some(block => block.type === BLOCK_TYPE.CODE)) {
      renderMap = renderMap.set(BLOCK_TYPE.CODE, {
        element: "code",
        wrapper: draftJs.DefaultDraftBlockRenderMap.get(BLOCK_TYPE.CODE).wrapper
      });
    }

    blockTypes.filter(block => block.element).forEach(block => {
      renderMap = renderMap.set(block.type, {
        element: block.element
      });
    });
    return renderMap;
  },

  /**
   * block style function automatically adding a class with the block's type.
   */
  blockStyleFn(block) {
    const type = block.getType();
    return `Draftail-block--${type} ${draftjsConductor.blockDepthStyleFn(block)}`;
  },

  /**
   * Configure key binding function from enabled blocks, styles, entities.
   */
  getKeyBindingFn(blockTypes, inlineStyles, entityTypes) {
    const getEnabled = activeTypes => activeTypes.reduce((enabled, type) => {
      enabled[type.type] = type.type;
      return enabled;
    }, {});

    const blocks = getEnabled(blockTypes);
    const styles = getEnabled(inlineStyles);
    const entities = getEnabled(entityTypes); // Emits key commands to use in `handleKeyCommand` in `Editor`.

    const keyBindingFn = e => {
      // Safeguard that we only trigger shortcuts with exact matches.
      // eg. cmd + shift + b should not trigger bold.
      if (e.shiftKey) {
        // Key bindings supported by Draft.js must be explicitely discarded.
        // See https://github.com/facebook/draft-js/issues/941.
        switch (e.keyCode) {
          case KEY_CODES.B:
            return undefined;

          case KEY_CODES.I:
            return undefined;

          case KEY_CODES.J:
            return undefined;

          case KEY_CODES.U:
            return undefined;

          case KEY_CODES.X:
            return hasCmd(e) && styles[INLINE_STYLE.STRIKETHROUGH];

          case KEY_CODES[7]:
            return hasCmd(e) && blocks[BLOCK_TYPE.ORDERED_LIST_ITEM];

          case KEY_CODES[8]:
            return hasCmd(e) && blocks[BLOCK_TYPE.UNORDERED_LIST_ITEM];

          default:
            return draftJs.getDefaultKeyBinding(e);
        }
      }

      const ctrlAlt = (e.ctrlKey || e.metaKey) && e.altKey;

      switch (e.keyCode) {
        case KEY_CODES.K:
          return hasCmd(e) && entities.LINK;

        case KEY_CODES.B:
          return hasCmd(e) && styles[INLINE_STYLE.BOLD];

        case KEY_CODES.I:
          return hasCmd(e) && styles[INLINE_STYLE.ITALIC];

        case KEY_CODES.J:
          return hasCmd(e) && styles[INLINE_STYLE.CODE];

        case KEY_CODES.U:
          return hasCmd(e) && styles[INLINE_STYLE.UNDERLINE];

        case KEY_CODES["."]:
          return hasCmd(e) && styles[INLINE_STYLE.SUPERSCRIPT];

        case KEY_CODES[","]:
          return hasCmd(e) && styles[INLINE_STYLE.SUBSCRIPT];

        case KEY_CODES[0]:
          // Reverting to unstyled block is always available.
          return ctrlAlt && BLOCK_TYPE.UNSTYLED;

        case KEY_CODES[1]:
          return ctrlAlt && blocks[BLOCK_TYPE.HEADER_ONE];

        case KEY_CODES[2]:
          return ctrlAlt && blocks[BLOCK_TYPE.HEADER_TWO];

        case KEY_CODES[3]:
          return ctrlAlt && blocks[BLOCK_TYPE.HEADER_THREE];

        case KEY_CODES[4]:
          return ctrlAlt && blocks[BLOCK_TYPE.HEADER_FOUR];

        case KEY_CODES[5]:
          return ctrlAlt && blocks[BLOCK_TYPE.HEADER_FIVE];

        case KEY_CODES[6]:
          return ctrlAlt && blocks[BLOCK_TYPE.HEADER_SIX];

        default:
          return draftJs.getDefaultKeyBinding(e);
      }
    };

    return keyBindingFn;
  },

  hasKeyboardShortcut(type) {
    return !!KEYBOARD_SHORTCUTS[type];
  },

  getKeyboardShortcut(type) {
    let isMacOS = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : IS_MAC_OS;
    const shortcut = KEYBOARD_SHORTCUTS[type];
    const system = isMacOS ? "macOS" : "other";
    return shortcut && shortcut[system] || shortcut;
  },

  /**
   * Defines whether a block should be altered to a new type when
   * the user types a given mark.
   * This powers the "autolist" feature.
   *
   * Returns the new block type, or false if no replacement should occur.
   */
  handleBeforeInputBlockType(mark, blockTypes) {
    return blockTypes.find(b => b.type === INPUT_BLOCK_MAP[mark]) ? INPUT_BLOCK_MAP[mark] : false;
  },

  handleBeforeInputHR(mark, block) {
    return mark === INPUT_ENTITY_MAP[ENTITY_TYPE.HORIZONTAL_RULE] && block.getType() !== BLOCK_TYPE.CODE;
  },

  /**
   * Checks whether a given input string contains style shortcuts.
   * If so, returns the range onto which the shortcut is applied.
   */
  handleBeforeInputInlineStyle(input, inlineStyles) {
    const activeShortcuts = INPUT_STYLE_MAP.filter(_ref => {
      let {
        type
      } = _ref;
      return inlineStyles.some(s => s.type === type);
    });
    let range;
    const match = activeShortcuts.find(_ref2 => {
      let {
        regex
      } = _ref2;
      // Re-create a RegExp object every time because RegExp is stateful.
      range = new RegExp(regex, "g").exec(input);
      return range;
    });
    return range && match ? {
      pattern: match.pattern,
      start: range.index === 0 ? 0 : range.index + 1,
      end: range.index + range[0].length,
      type: match.type
    } : false;
  },

  getCustomStyleMap(inlineStyles) {
    const customStyleMap = {};
    inlineStyles.forEach(style => {
      if (style.style) {
        customStyleMap[style.type] = style.style;
      } else if (CUSTOM_STYLE_MAP[style.type]) {
        customStyleMap[style.type] = CUSTOM_STYLE_MAP[style.type];
      } else {
        customStyleMap[style.type] = {};
      }
    });
    return customStyleMap;
  },

  /**
   * Applies whitelist and blacklist operations to the editor content,
   * so the resulting editor state is shaped according to Draftail
   * expectations and configuration.
   */
  filterPaste(_ref3, editorState) {
    let {
      maxListNesting,
      enableHorizontalRule,
      enableLineBreak,
      blockTypes,
      inlineStyles,
      entityTypes
    } = _ref3;
    const enabledEntityTypes = entityTypes.slice();
    const whitespacedCharacters = ["\t", "📷"];

    if (enableHorizontalRule) {
      enabledEntityTypes.push({
        type: ENTITY_TYPE.HORIZONTAL_RULE
      });
    }

    if (!enableLineBreak) {
      whitespacedCharacters.push("\n");
    }

    return draftjsFilters.filterEditorState({
      blocks: blockTypes.map(b => b.type),
      styles: inlineStyles.map(s => s.type),
      entities: enabledEntityTypes,
      maxNesting: maxListNesting,
      whitespacedCharacters
    }, editorState);
  }

};

/**
 * Icon as SVG element. Can optionally render a React element instead.
 */
const Icon = _ref => {
  let {
    icon,
    title,
    className
  } = _ref;
  let children;

  if (typeof icon === "string") {
    if (icon.includes("#")) {
      children = /*#__PURE__*/React__default['default'].createElement("use", {
        xlinkHref: icon
      });
    } else {
      children = /*#__PURE__*/React__default['default'].createElement("path", {
        d: icon
      });
    }
  } else if (Array.isArray(icon)) {
    // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
    children = icon.map((d, i) => /*#__PURE__*/React__default['default'].createElement("path", {
      key: i,
      d: d
    }));
  } else {
    return icon;
  }

  return /*#__PURE__*/React__default['default'].createElement("svg", {
    width: "16",
    height: "16",
    viewBox: "0 0 1024 1024",
    className: `Draftail-Icon ${className || ""}`,
    "aria-hidden": title ? null : true,
    role: title ? "img" : null,
    "aria-label": title || null
  }, children);
};

Icon.defaultProps = {
  title: null,
  className: null
};

/**
 * Displays a basic button, with optional active variant,
 * enriched with a tooltip. The tooltip stops showing on click.
 */
class ToolbarButton extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      showTooltipOnHover: true
    };
    this.onMouseDown = this.onMouseDown.bind(this);
    this.onMouseLeave = this.onMouseLeave.bind(this);
  }
  /* :: onMouseDown: (e: Event) => void; */


  onMouseDown(e) {
    const {
      name,
      onClick
    } = this.props;
    e.preventDefault();
    this.setState({
      showTooltipOnHover: false
    });

    if (onClick) {
      onClick(name || "");
    }
  }
  /* :: onMouseLeave: () => void; */


  onMouseLeave() {
    this.setState({
      showTooltipOnHover: true
    });
  }

  render() {
    const {
      name,
      active,
      label,
      title,
      icon
    } = this.props;
    const {
      showTooltipOnHover
    } = this.state;
    return /*#__PURE__*/React__default['default'].createElement("button", {
      name: name,
      className: `Draftail-ToolbarButton${active ? " Draftail-ToolbarButton--active" : ""}`,
      type: "button",
      "aria-label": title || null,
      "data-draftail-balloon": title && showTooltipOnHover ? true : null,
      tabIndex: -1,
      onMouseDown: this.onMouseDown,
      onMouseLeave: this.onMouseLeave
    }, typeof icon !== "undefined" && icon !== null ? /*#__PURE__*/React__default['default'].createElement(Icon, {
      icon: icon
    }) : null, label ? /*#__PURE__*/React__default['default'].createElement("span", {
      className: "Draftail-ToolbarButton__label"
    }, label) : null);
  }

}

ToolbarButton.defaultProps = {
  name: null,
  active: false,
  label: null,
  title: null,
  icon: null,
  onClick: null
};

const ToolbarGroup = _ref => {
  let {
    children
  } = _ref;
  const hasChildren = React__default['default'].Children.toArray(children).some(c => c !== null);
  return hasChildren ? /*#__PURE__*/React__default['default'].createElement("div", {
    className: "Draftail-ToolbarGroup"
  }, children) : null;
};

ToolbarGroup.defaultProps = {
  children: null
};

const getButtonLabel = (type, config) => {
  const icon = typeof config === "boolean" ? undefined : config.icon;

  if (typeof config.label === "string" || config.label === null) {
    return config.label;
  }

  if (typeof icon !== "undefined") {
    return null;
  }

  return LABELS[type];
};
const showButton = config => Boolean(config.icon) || Boolean(getButtonLabel(config.type, config));
const getButtonTitle = (type, config) => {
  const description = typeof config === "boolean" || typeof config.description === "undefined" ? DESCRIPTIONS[type] : config.description;
  const hasShortcut = behavior.hasKeyboardShortcut(type);
  let title = description;

  if (hasShortcut) {
    const desc = description ? `${description}\n` : "";
    title = `${desc}${behavior.getKeyboardShortcut(type)}`;
  }

  return title;
};

class ToolbarDefaults extends React.PureComponent {
  render() {
    const {
      currentStyles,
      currentBlock,
      blockTypes,
      inlineStyles,
      enableHorizontalRule,
      enableLineBreak,
      showUndoControl,
      showRedoControl,
      entityTypes,
      toggleBlockType,
      toggleInlineStyle,
      addHR,
      addBR,
      onUndoRedo,
      onRequestSource
    } = this.props;
    return [/*#__PURE__*/React__default['default'].createElement(ToolbarGroup, {
      key: "styles"
    }, inlineStyles.filter(showButton).map(t => /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      key: t.type,
      name: t.type,
      active: currentStyles.has(t.type),
      label: getButtonLabel(t.type, t),
      title: getButtonTitle(t.type, t),
      icon: t.icon,
      onClick: toggleInlineStyle
    }))), /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, {
      key: "blocks"
    }, blockTypes.filter(showButton).map(t => /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      key: t.type,
      name: t.type,
      active: currentBlock === t.type,
      label: getButtonLabel(t.type, t),
      title: getButtonTitle(t.type, t),
      icon: t.icon,
      onClick: toggleBlockType
    }))), /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, {
      key: "hr-br"
    }, enableHorizontalRule ? /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      name: ENTITY_TYPE.HORIZONTAL_RULE,
      onClick: addHR,
      label: getButtonLabel(ENTITY_TYPE.HORIZONTAL_RULE, enableHorizontalRule),
      title: getButtonTitle(ENTITY_TYPE.HORIZONTAL_RULE, enableHorizontalRule),
      icon: typeof enableHorizontalRule !== "boolean" ? enableHorizontalRule.icon : null
    }) : null, enableLineBreak ? /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      name: BR_TYPE,
      onClick: addBR,
      label: getButtonLabel(BR_TYPE, enableLineBreak),
      title: getButtonTitle(BR_TYPE, enableLineBreak),
      icon: typeof enableLineBreak !== "boolean" ? enableLineBreak.icon : null
    }) : null), /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, {
      key: "entities"
    }, entityTypes.filter(showButton).map(t => /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      key: t.type,
      name: t.type,
      onClick: onRequestSource,
      label: getButtonLabel(t.type, t),
      title: getButtonTitle(t.type, t),
      icon: t.icon
    }))), /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, {
      key: "undo-redo"
    }, showUndoControl ? /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      name: UNDO_TYPE,
      onClick: onUndoRedo,
      label: getButtonLabel(UNDO_TYPE, showUndoControl),
      title: getButtonTitle(UNDO_TYPE, showUndoControl),
      icon: typeof showUndoControl !== "boolean" ? showUndoControl.icon : null
    }) : null, showRedoControl ? /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      name: REDO_TYPE,
      onClick: onUndoRedo,
      label: getButtonLabel(REDO_TYPE, showRedoControl),
      title: getButtonTitle(REDO_TYPE, showRedoControl),
      icon: typeof showRedoControl !== "boolean" ? showRedoControl.icon : null
    }) : null)];
  }

}

const Toolbar = props => {
  const {
    controls,
    getEditorState,
    onChange
  } = props;
  return /*#__PURE__*/React__default['default'].createElement("div", {
    className: "Draftail-Toolbar",
    role: "toolbar"
  }, /*#__PURE__*/React__default['default'].createElement(ToolbarDefaults, props), /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, null, controls.map((Control, i) => /*#__PURE__*/React__default['default'].createElement(Control // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
  , {
    key: i,
    getEditorState: getEditorState,
    onChange: onChange
  }))));
};

/**
 * An <hr/> in the editor.
 */

const DividerBlock = () => /*#__PURE__*/React__default['default'].createElement("hr", {
  className: "Draftail-DividerBlock"
});

const defaultProps = {
  /** Initial content of the editor. Use this to edit pre-existing content. */
  rawContentState: null,

  /** Called when changes occurred. Use this to persist editor content. */
  onSave: null,

  /** Content of the editor, when using the editor as a controlled component. Incompatible with `rawContentState` and `onSave`. */
  editorState: null,

  /** Called whenever the editor state is updated. Use this to manage the content of a controlled editor. Incompatible with `rawContentState` and `onSave`. */
  onChange: null,

  /** Called when the editor receives focus. */
  onFocus: null,

  /** Called when the editor loses focus. */
  onBlur: null,

  /** Displayed when the editor is empty. Hidden if the user changes styling. */
  placeholder: null,

  /** Enable the use of horizontal rules in the editor. */
  enableHorizontalRule: false,

  /** Enable the use of line breaks in the editor. */
  enableLineBreak: false,

  /** Show undo control in the toolbar. */
  showUndoControl: false,

  /** Show redo control in the toolbar. */
  showRedoControl: false,

  /** Disable copy/paste of rich text in the editor. */
  stripPastedStyles: true,

  /** Set whether spellcheck is turned on for your editor.
   * See https://draftjs.org/docs/api-reference-editor.html#spellcheck.
   */
  spellCheck: false,

  /** Set whether the editor should be rendered in readOnly mode.
   * See https://draftjs.org/docs/api-reference-editor.html#readonly
   */
  readOnly: false,

  /** Optionally set the overriding text alignment for this editor.
   * See https://draftjs.org/docs/api-reference-editor.html#textalignment.
   */
  textAlignment: null,

  /** Optionally set the overriding text directionality for this editor.
   * See https://draftjs.org/docs/api-reference-editor.html#textdirectionality.
   */
  textDirectionality: null,

  /** Set if auto capitalization is turned on and how it behaves.
   * See https://draftjs.org/docs/api-reference-editor.html#autocapitalize-string.
   */
  autoCapitalize: null,

  /** Set if auto complete is turned on and how it behaves.
   * See https://draftjs.org/docs/api-reference-editor.html#autocomplete-string.
   */
  autoComplete: null,

  /** Set if auto correct is turned on and how it behaves.
   * See https://draftjs.org/docs/api-reference-editor.html#autocorrect-string.
   */
  autoCorrect: null,

  /** See https://draftjs.org/docs/api-reference-editor.html#aria-props. */
  ariaDescribedBy: null,

  /** List of the available block types. */
  blockTypes: [],

  /** List of the available inline styles. */
  inlineStyles: [],

  /** List of the available entity types. */
  entityTypes: [],

  /** List of active decorators. */
  decorators: [],

  /** List of extra toolbar controls. */
  controls: [],

  /** List of plugins of the draft-js-plugins architecture. */
  plugins: [],

  /** Optionally override the default Draftail toolbar, removing or replacing it. */
  topToolbar: Toolbar,

  /** Optionally add a custom toolbar underneath the editor, e.g. for metrics. */
  bottomToolbar: null,

  /** Max level of nesting for list items. 0 = no nesting. Maximum = 10. */
  maxListNesting: 1,

  /** Frequency at which to call the onSave callback (ms). */
  stateSaveInterval: 250
};

/* :: import type { ElementRef, Node } from "react"; */

/**
 * Main component of the Draftail editor.
 * Contains the Draft.js editor instance, and ties together UI and behavior.
 */
class DraftailEditor extends React.Component {
  /* :: editorRef: ElementRef<Editor>; */

  /* :: copySource: { unregister: () => void }; */

  /* :: updateTimeout: ?number; */

  /* :: lockEditor: () => void; */

  /* :: unlockEditor: () => void; */

  /* :: getEditorState: () => EditorState; */
  constructor(props) {
    super(props);
    this.onChange = this.onChange.bind(this);
    this.saveState = this.saveState.bind(this);
    this.toggleSource = this.toggleSource.bind(this);
    this.toggleEditor = this.toggleEditor.bind(this);
    this.lockEditor = this.toggleEditor.bind(this, true);
    this.unlockEditor = this.toggleEditor.bind(this, false);
    this.handleReturn = this.handleReturn.bind(this);
    this.onFocus = this.onFocus.bind(this);
    this.onBlur = this.onBlur.bind(this);
    this.onTab = this.onTab.bind(this);
    this.handleKeyCommand = this.handleKeyCommand.bind(this);
    this.handleBeforeInput = this.handleBeforeInput.bind(this);
    this.handlePastedText = this.handlePastedText.bind(this);
    this.toggleBlockType = this.toggleBlockType.bind(this);
    this.toggleInlineStyle = this.toggleInlineStyle.bind(this);
    this.onEditEntity = this.onEditEntity.bind(this);
    this.onRemoveEntity = this.onRemoveEntity.bind(this);
    this.addHR = this.addHR.bind(this);
    this.addBR = this.addBR.bind(this);
    this.onUndoRedo = this.onUndoRedo.bind(this);
    this.blockRenderer = this.blockRenderer.bind(this);
    this.onRequestSource = this.onRequestSource.bind(this);
    this.onCompleteSource = this.onCompleteSource.bind(this);
    this.onCloseSource = this.onCloseSource.bind(this);
    this.focus = this.focus.bind(this);
    this.renderSource = this.renderSource.bind(this);
    const {
      editorState,
      rawContentState
    } = props;
    this.state = {
      readOnlyState: false,
      hasFocus: false,
      sourceOptions: null
    };

    if (editorState !== null) {
      this.getEditorState = this.getEditorStateProp.bind(this);
    } else {
      // If editorState is not used as a prop, create it in local state from rawContentState.
      this.state.editorState = draftjsConductor.createEditorStateFromRaw(rawContentState);
      this.getEditorState = this.getEditorStateState.bind(this);
    }
  }

  componentDidMount() {
    this.copySource = draftjsConductor.registerCopySource(this.editorRef.editor);
  }

  componentWillUnmount() {
    this.copySource.unregister();
    window.clearTimeout(this.updateTimeout);
  }
  /* :: onFocus: () => void; */


  onFocus() {
    this.setState({
      hasFocus: true
    });
    const {
      onFocus
    } = this.props;

    if (onFocus) {
      onFocus();
    }
  }
  /* :: onBlur: () => void; */


  onBlur() {
    this.setState({
      hasFocus: false
    });
    const {
      onBlur
    } = this.props;

    if (onBlur) {
      onBlur();
    }
  }
  /* :: onTab: (event: SyntheticKeyboardEvent<>) => true; */


  onTab(event) {
    const {
      maxListNesting
    } = this.props;
    const editorState = this.getEditorState();
    const newState = draftJs.RichUtils.onTab(event, editorState, maxListNesting);
    this.onChange(newState);
    return true;
  }
  /* :: onChange: (nextState: EditorState) => void; */


  onChange(nextState) {
    const {
      stateSaveInterval,
      maxListNesting,
      enableHorizontalRule,
      enableLineBreak,
      blockTypes,
      inlineStyles,
      entityTypes,
      onChange
    } = this.props;
    const editorState = this.getEditorState();
    const shouldFilterPaste = nextState.getCurrentContent() !== editorState.getCurrentContent() && nextState.getLastChangeType() === "insert-fragment";
    let filteredState = nextState;

    if (shouldFilterPaste) {
      filteredState = behavior.filterPaste({
        maxListNesting,
        enableHorizontalRule,
        enableLineBreak,
        blockTypes,
        inlineStyles,
        entityTypes
      }, filteredState);
    }

    if (onChange) {
      onChange(filteredState);
    } else {
      this.setState({
        editorState: filteredState
      }, () => {
        window.clearTimeout(this.updateTimeout);
        this.updateTimeout = window.setTimeout(this.saveState, stateSaveInterval);
      });
    }
  }
  /* :: onEditEntity: (entityKey: string) => void; */


  onEditEntity(entityKey) {
    const {
      entityTypes
    } = this.props;
    const editorState = this.getEditorState();
    const content = editorState.getCurrentContent();
    const entity = content.getEntity(entityKey);
    const entityType = entityTypes.find(t => t.type === entity.type); // $FlowFixMe

    if (!entityType.block) {
      const entitySelection = DraftUtils.getEntitySelection(editorState, entityKey);
      const nextState = draftJs.EditorState.acceptSelection(editorState, entitySelection);
      this.onChange(nextState);
    }

    this.toggleSource(entity.getType(), entityKey, entity);
  }
  /* :: onRemoveEntity: (entityKey: string, blockKey: string) => void; */


  onRemoveEntity(entityKey, blockKey) {
    const {
      entityTypes
    } = this.props;
    const editorState = this.getEditorState();
    const content = editorState.getCurrentContent();
    const entity = content.getEntity(entityKey);
    const entityType = entityTypes.find(t => t.type === entity.type);
    let newState = editorState; // $FlowFixMe

    if (entityType.block) {
      newState = DraftUtils.removeBlockEntity(newState, entityKey, blockKey);
    } else {
      const entitySelection = DraftUtils.getEntitySelection(editorState, entityKey);
      newState = draftJs.RichUtils.toggleLink(newState, entitySelection, null);
    }

    this.onChange(newState);
  }
  /* :: onUndoRedo: (type: string) => void; */


  onUndoRedo(type) {
    const editorState = this.getEditorState();
    let newEditorState = editorState;

    if (type === UNDO_TYPE) {
      newEditorState = draftJs.EditorState.undo(editorState);
    } else if (type === REDO_TYPE) {
      newEditorState = draftJs.EditorState.redo(editorState);
    }

    this.onChange(newEditorState);
  }
  /* :: onRequestSource: (entityType: string) => void; */


  onRequestSource(entityType) {
    const editorState = this.getEditorState();
    const contentState = editorState.getCurrentContent();
    const entityKey = DraftUtils.getSelectionEntity(editorState);
    this.toggleSource(entityType, entityKey, entityKey ? contentState.getEntity(entityKey) : null);
  }
  /* :: onCompleteSource: (nextState: EditorState) => void; */


  onCompleteSource(nextState) {
    this.setState({
      sourceOptions: null
    }, () => {
      if (nextState) {
        this.onChange(nextState);
      }

      window.setTimeout(() => {
        this.setState({
          readOnlyState: false
        }, () => {
          window.setTimeout(() => {
            this.focus();
          }, 0);
        });
      }, 0);
    });
  }
  /* :: onCloseSource: () => void; */


  onCloseSource() {
    this.setState({
      sourceOptions: null,
      readOnlyState: false
    });
  }
  /* :: getEditorStateProp: () => EditorState; */


  getEditorStateProp() {
    const {
      editorState
    } = this.props;
    return editorState;
  }
  /* :: getEditorStateState: () => EditorState; */


  getEditorStateState() {
    const {
      editorState
    } = this.state;
    return editorState;
  }
  /* :: saveState: () => void; */


  saveState() {
    const {
      onSave
    } = this.props;
    const editorState = this.getEditorState();

    if (onSave) {
      onSave(draftjsConductor.serialiseEditorStateToRaw(editorState));
    }
  }
  /* :: toggleEditor: (readOnlyState: boolean) => void; */


  toggleEditor(readOnlyState) {
    this.setState({
      readOnlyState
    });
  }
  /* :: toggleSource: (type:string, entityKey: ?string, entity: ?EntityInstance) => void; */


  toggleSource(type, entityKey, entity) {
    const {
      entityTypes
    } = this.props;
    const entityType = entityTypes.find(item => item.type === type);
    this.setState({
      readOnlyState: true,
      sourceOptions: {
        entity,
        entityKey,
        entityType
      }
    });
  }
  /* :: handleReturn: (e: SyntheticKeyboardEvent<>) => 'not-handled' | 'handled'; */


  handleReturn(e) {
    const {
      enableLineBreak,
      inlineStyles
    } = this.props;
    const editorState = this.getEditorState();
    let ret = NOT_HANDLED; // alt + enter opens links and other entities with a `url` property.

    if (e.altKey) {
      // Mark the return as handled even if there is no entity.
      // alt + enter should never create a newline anyway.
      ret = HANDLED;
      const entityKey = DraftUtils.getSelectionEntity(editorState);

      if (entityKey) {
        const content = editorState.getCurrentContent();
        const entityData = content.getEntity(entityKey).getData();

        if (entityData.url) {
          window.open(entityData.url);
        }
      }
    } else {
      if (!enableLineBreak) {
        // Quick hack to disable soft line breaks.
        e.which = 0;
      }

      let newState = editorState;
      let newStyle = false;
      const selection = newState.getSelection(); // Check whether we should apply a Markdown styles shortcut.

      if (selection.isCollapsed()) {
        const block = DraftUtils.getSelectedBlock(editorState);
        newStyle = behavior.handleBeforeInputInlineStyle(block.getText(), inlineStyles);

        if (newStyle) {
          newState = DraftUtils.applyMarkdownStyle(newState, newStyle, "");
        }
      }

      const newLineState = DraftUtils.handleNewLine(newState, e); // Manually handle the return if there is a style to apply.

      if (!newLineState && newStyle) {
        const content = newState.getCurrentContent();
        const newContent = draftJs.Modifier.splitBlock(content, selection);
        newState = draftJs.EditorState.push(newState, newContent, "split-block"); // Do not propagate the style from the last block.

        newState = draftJs.RichUtils.toggleInlineStyle(newState, newStyle.type);
      } else {
        newState = newLineState;
      }

      if (newState && newState !== editorState) {
        ret = HANDLED;
        this.onChange(newState);
      }
    }

    return ret;
  }
  /* :: handleKeyCommand: (command: DraftEditorCommand) => 'handled' | 'not-handled'; */


  handleKeyCommand(command) {
    const {
      entityTypes,
      blockTypes,
      inlineStyles
    } = this.props;
    const editorState = this.getEditorState();

    if (entityTypes.some(t => t.type === command)) {
      this.onRequestSource(command);
      return HANDLED;
    }

    if (blockTypes.some(t => t.type === command)) {
      this.toggleBlockType(command);
      return HANDLED;
    }

    if (inlineStyles.some(t => t.type === command)) {
      this.toggleInlineStyle(command);
      return HANDLED;
    } // Special case – some delete commands on atomic blocks are not covered by RichUtils.


    if (command === "delete") {
      const newState = DraftUtils.handleDeleteAtomic(editorState);

      if (newState) {
        this.onChange(newState);
        return HANDLED;
      }
    } // If the command is known but not whitelisted for this editor, treat it as handled but don't do anything.


    if (KEY_COMMANDS.includes(command)) {
      return HANDLED;
    }

    const newState = draftJs.RichUtils.handleKeyCommand(editorState, command);

    if (newState) {
      this.onChange(newState);
      return HANDLED;
    }

    return NOT_HANDLED;
  }
  /* :: handleBeforeInput: (char: string) => 'handled' | 'not-handled'; */


  handleBeforeInput(char) {
    const {
      blockTypes,
      inlineStyles,
      enableHorizontalRule
    } = this.props;
    const editorState = this.getEditorState();
    const selection = editorState.getSelection();

    if (selection.isCollapsed()) {
      const block = DraftUtils.getSelectedBlock(editorState);
      const startOffset = selection.getStartOffset();
      const text = block.getText();
      const beforeInput = text.slice(0, startOffset);
      const mark = `${beforeInput}${char}`;
      let newEditorState = editorState;
      const newBlockType = behavior.handleBeforeInputBlockType(mark, blockTypes);

      if (newBlockType) {
        newEditorState = DraftUtils.resetBlockWithType(newEditorState, newBlockType, text.replace(beforeInput, ""));
      }

      if (enableHorizontalRule && behavior.handleBeforeInputHR(mark, block)) {
        newEditorState = DraftUtils.removeBlock(DraftUtils.addHorizontalRuleRemovingSelection(newEditorState), block.getKey());
      }

      const newStyle = behavior.handleBeforeInputInlineStyle(beforeInput, inlineStyles);

      if (newStyle) {
        newEditorState = DraftUtils.applyMarkdownStyle(newEditorState, newStyle, char);
      }

      if (newEditorState !== editorState) {
        this.onChange(newEditorState);
        return HANDLED;
      }
    }

    return NOT_HANDLED;
  }
  /* :: handlePastedText: (text: string, html: ?string, editorState: EditorState) => 'handled' | 'not-handled'; */


  handlePastedText(text, html, editorState) {
    const {
      stripPastedStyles,
      entityTypes
    } = this.props; // Leave paste handling to Draft.js when stripping styles is desirable.

    if (stripPastedStyles) {
      return NOT_HANDLED;
    }

    const hasProcessedEntity = entityTypes.some(t => {
      const ret = t.onPaste && t.onPaste(text, html, editorState, {
        setEditorState: this.onChange,
        getEditorState: this.getEditorState
      }, t);
      return ret === HANDLED;
    });

    if (hasProcessedEntity) {
      return HANDLED;
    }

    const pastedState = draftjsConductor.handleDraftEditorPastedText(html, editorState);

    if (pastedState) {
      this.onChange(pastedState);
      return HANDLED;
    }

    return NOT_HANDLED;
  }
  /* :: toggleBlockType: (blockType: string) => void; */


  toggleBlockType(blockType) {
    const editorState = this.getEditorState();
    this.onChange(draftJs.RichUtils.toggleBlockType(editorState, blockType));
  }
  /* :: toggleInlineStyle: (inlineStyle: string) => void; */


  toggleInlineStyle(inlineStyle) {
    const editorState = this.getEditorState();
    this.onChange(draftJs.RichUtils.toggleInlineStyle(editorState, inlineStyle));
  }
  /* :: addHR: () => void; */


  addHR() {
    const editorState = this.getEditorState();
    this.onChange(DraftUtils.addHorizontalRuleRemovingSelection(editorState));
  }
  /* :: addBR: () => void; */


  addBR() {
    const editorState = this.getEditorState();
    this.onChange(DraftUtils.addLineBreak(editorState));
  }
  /* :: blockRenderer: (block: ContentBlock) => {}; */


  blockRenderer(block) {
    const {
      entityTypes
    } = this.props;
    const editorState = this.getEditorState();
    const contentState = editorState.getCurrentContent();

    if (block.getType() !== BLOCK_TYPE.ATOMIC) {
      return null;
    }

    const entityKey = block.getEntityAt(0);

    if (!entityKey) {
      return {
        editable: false
      };
    }

    const entity = contentState.getEntity(entityKey);
    const isHorizontalRule = entity.type === ENTITY_TYPE.HORIZONTAL_RULE;

    if (isHorizontalRule) {
      return {
        component: DividerBlock,
        editable: false
      };
    }

    const entityType = entityTypes.find(t => t.type === entity.type);
    return {
      // $FlowFixMe
      component: entityType.block,
      editable: false,
      props: {
        /** The editorState is available for arbitrary content manipulation. */
        editorState,

        /** Current entity to manage. */
        entity,

        /** Current entityKey to manage. */
        entityKey,

        /** Whole entityType configuration, as provided to the editor. */
        entityType,

        /** Make the whole editor read-only, except for the block. */
        lockEditor: this.lockEditor,

        /** Make the editor editable again. */
        unlockEditor: this.unlockEditor,

        /** Shorthand to edit entity data. */
        onEditEntity: this.onEditEntity.bind(null, entityKey),

        /** Shorthand to remove an entity, and the related block. */
        onRemoveEntity: this.onRemoveEntity.bind(null, entityKey, block.getKey()),

        /** Update the editorState with arbitrary changes. */
        onChange: this.onChange
      }
    };
  }
  /**
   * Imperative focus API similar to that of Draft.js.
   * See https://draftjs.org/docs/advanced-topics-managing-focus.html#content.
   */

  /* :: focus: () => void; */


  focus() {
    this.editorRef.focus();
  }
  /* :: renderSource: () => ?Node; */


  renderSource() {
    const {
      sourceOptions
    } = this.state;
    const editorState = this.getEditorState();

    if (sourceOptions && sourceOptions.entityType) {
      const Source = sourceOptions.entityType.source;
      return /*#__PURE__*/React__default['default'].createElement(Source
      /** The editorState is available for arbitrary content manipulation. */
      , {
        editorState: editorState
        /** Takes the updated editorState, or null if there are no changes, and focuses the editor. */
        ,
        onComplete: this.onCompleteSource
        /** Closes the source, without focusing the editor again. */
        ,
        onClose: this.onCloseSource
        /** Current entity to edit, if any. */
        ,
        entity: sourceOptions.entity
        /** Current entityKey to edit, if any. */
        ,
        entityKey: sourceOptions.entityKey
        /** Whole entityType configuration, as provided to the editor. */
        ,
        entityType: sourceOptions.entityType
      });
    }

    return null;
  }

  render() {
    const {
      placeholder,
      enableHorizontalRule,
      enableLineBreak,
      showUndoControl,
      showRedoControl,
      stripPastedStyles,
      spellCheck,
      textAlignment,
      textDirectionality,
      autoCapitalize,
      autoComplete,
      autoCorrect,
      ariaDescribedBy,
      blockTypes,
      inlineStyles,
      entityTypes,
      decorators,
      controls,
      readOnly,
      maxListNesting,
      plugins,
      topToolbar,
      bottomToolbar
    } = this.props;
    const {
      hasFocus,
      readOnlyState
    } = this.state;
    const editorState = this.getEditorState();
    const isReadOnly = readOnlyState || readOnly;
    const hidePlaceholder = DraftUtils.shouldHidePlaceholder(editorState);
    const entityDecorators = entityTypes.filter(type => !!type.decorator).map(type => ({
      strategy: DraftUtils.getEntityTypeStrategy(type.type),
      // $FlowFixMe
      component: decorateComponentWithProps__default['default'](type.decorator, {
        onEdit: this.onEditEntity,
        onRemove: this.onRemoveEntity
      })
    }));
    const TopToolbar = topToolbar;
    const BottomToolbar = bottomToolbar;
    const toolbarProps = {
      currentStyles: editorState.getCurrentInlineStyle(),
      currentBlock: DraftUtils.getSelectedBlock(editorState).getType(),
      enableHorizontalRule,
      enableLineBreak,
      showUndoControl,
      showRedoControl,
      blockTypes,
      inlineStyles,
      entityTypes,
      controls,
      readOnly: isReadOnly,
      toggleBlockType: this.toggleBlockType,
      toggleInlineStyle: this.toggleInlineStyle,
      addHR: this.addHR,
      addBR: this.addBR,
      onUndoRedo: this.onUndoRedo,
      onRequestSource: this.onRequestSource,
      getEditorState: this.getEditorState,
      onChange: this.onChange
    };
    return /*#__PURE__*/React__default['default'].createElement("div", {
      className: `Draftail-Editor${isReadOnly ? " Draftail-Editor--readonly" : ""}${hidePlaceholder ? " Draftail-Editor--hide-placeholder" : ""}${hasFocus ? " Draftail-Editor--focus" : ""}`
    }, TopToolbar ? /*#__PURE__*/React__default['default'].createElement(TopToolbar, toolbarProps) : null, /*#__PURE__*/React__default['default'].createElement(Editor__default['default'], {
      customStyleMap: behavior.getCustomStyleMap(inlineStyles),
      ref: ref => {
        this.editorRef = ref;
      },
      editorState: editorState,
      onChange: this.onChange,
      placeholder: placeholder,
      readOnly: isReadOnly,
      stripPastedStyles: stripPastedStyles,
      spellCheck: spellCheck,
      textAlignment: textAlignment,
      textDirectionality: textDirectionality,
      autoCapitalize: autoCapitalize,
      autoComplete: autoComplete,
      autoCorrect: autoCorrect,
      ariaDescribedBy: ariaDescribedBy,
      handleReturn: this.handleReturn,
      defaultKeyBindings: false,
      handleKeyCommand: this.handleKeyCommand,
      handleBeforeInput: this.handleBeforeInput,
      handlePastedText: this.handlePastedText,
      onFocus: this.onFocus,
      onBlur: this.onBlur,
      onTab: this.onTab,
      blockRendererFn: this.blockRenderer,
      blockRenderMap: behavior.getBlockRenderMap(blockTypes),
      blockStyleFn: behavior.blockStyleFn // Include the keyBindingFn in a plugin here so that
      // other plugin keyBindingFn's are still called, while
      // still being able to override the Draft.js oversensitive
      // keyboard shortcuts.
      ,
      plugins: plugins.concat([{
        keyBindingFn: behavior.getKeyBindingFn(blockTypes, inlineStyles, entityTypes)
      }]) // $FlowFixMe
      ,
      decorators: decorators.concat(entityDecorators)
    }), BottomToolbar ? /*#__PURE__*/React__default['default'].createElement(BottomToolbar, toolbarProps) : null, this.renderSource(), /*#__PURE__*/React__default['default'].createElement(draftjsConductor.ListNestingStyles, {
      max: maxListNesting
    }));
  }

}

DraftailEditor.defaultProps = defaultProps;

const getButtonLabel$1 = (type, config) => {
  const icon = typeof config === "boolean" ? undefined : config.icon;

  if (typeof config.label === "string" || config.label === null) {
    return config.label;
  }

  if (typeof icon !== "undefined") {
    return null;
  }

  return LABELS[type];
};

const showButton$1 = config => Boolean(config.icon) || Boolean(getButtonLabel$1(config.type, config));

const showEntityButton = config => showButton$1(config) && Boolean(config.decorator);

const getButtonTitle$1 = (type, config) => {
  const description = typeof config === "boolean" || typeof config.description === "undefined" ? DESCRIPTIONS[type] : config.description;
  const hasShortcut = behavior.hasKeyboardShortcut(type);
  let title = description;

  if (hasShortcut) {
    const desc = description ? `${description}\n` : "";
    title = `${desc}${behavior.getKeyboardShortcut(type)}`;
  }

  return title;
};

class ToolbarDefaults$1 extends React.PureComponent {
  render() {
    const {
      currentStyles,
      currentBlock,
      blockTypes,
      inlineStyles,
      entityTypes,
      toggleBlockType,
      toggleInlineStyle,
      onRequestSource
    } = this.props;
    return [/*#__PURE__*/React__default['default'].createElement(ToolbarGroup, {
      key: "styles"
    }, inlineStyles.filter(showButton$1).map(t => /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      key: t.type,
      name: t.type,
      active: currentStyles.has(t.type),
      label: getButtonLabel$1(t.type, t),
      title: getButtonTitle$1(t.type, t),
      icon: t.icon,
      onClick: toggleInlineStyle
    }))), /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, {
      key: "blocks"
    }, blockTypes.filter(showButton$1).map(t => /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      key: t.type,
      name: t.type,
      active: currentBlock === t.type,
      label: getButtonLabel$1(t.type, t),
      title: getButtonTitle$1(t.type, t),
      icon: t.icon,
      onClick: toggleBlockType
    }))), /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, {
      key: "entities"
    }, entityTypes.filter(showEntityButton).map(t => /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
      key: t.type,
      name: t.type,
      onClick: onRequestSource,
      label: getButtonLabel$1(t.type, t),
      title: getButtonTitle$1(t.type, t),
      icon: t.icon
    })))];
  }

}

const getReferenceClientRect = () => draftJs.getVisibleSelectionRect(window);

const InlineToolbar = props => {
  const {
    controls,
    getEditorState,
    onChange
  } = props;
  const tippyParentRef = React.useRef(null);
  const editorState = getEditorState();
  const selection = editorState.getSelection();
  const isVisible = selection.getHasFocus() && !selection.isCollapsed() && Boolean(getReferenceClientRect()); // if (!isVisible) {
  //   return null;
  // }

  return /*#__PURE__*/React__default['default'].createElement(React__default['default'].Fragment, null, isVisible ? /*#__PURE__*/React__default['default'].createElement(Tippy__default['default'], {
    visible: isVisible,
    getReferenceClientRect: getReferenceClientRect,
    maxWidth: "100%",
    interactive: true,
    appendTo: () => tippyParentRef.current,
    content: /*#__PURE__*/React__default['default'].createElement("div", {
      className: "Draftail-Toolbar",
      role: "toolbar"
    }, /*#__PURE__*/React__default['default'].createElement(ToolbarDefaults$1, props), /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, null, controls.filter(control => Boolean(control.inline)).map((control, i) => {
      const Control = control.inline;
      return /*#__PURE__*/React__default['default'].createElement(Control // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
      , {
        key: i,
        getEditorState: getEditorState,
        onChange: onChange
      });
    })))
  }) : null, /*#__PURE__*/React__default['default'].createElement("div", {
    ref: tippyParentRef
  }));
};

const MetaToolbar = _ref => {
  let {
    showBlockEntities,
    entityTypes,
    controls,
    getEditorState,
    onChange,
    onRequestSource
  } = _ref;
  return /*#__PURE__*/React__default['default'].createElement("div", {
    className: "Draftail-MetaToolbar"
  }, showBlockEntities ? /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, {
    key: "entities"
  }, entityTypes.filter(entityType => showButton(entityType) && entityType.block).map(t => /*#__PURE__*/React__default['default'].createElement(ToolbarButton, {
    key: t.type,
    name: t.type,
    onClick: onRequestSource,
    label: getButtonLabel(t.type, t),
    title: getButtonTitle(t.type, t),
    icon: t.icon
  }))) : null, /*#__PURE__*/React__default['default'].createElement(ToolbarGroup, null, controls.map((control, i) => {
    if (control.inline || control.block) {
      return null;
    }

    const Control = control.meta || control;
    return /*#__PURE__*/React__default['default'].createElement(Control // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
    , {
      key: i,
      getEditorState: getEditorState,
      onChange: onChange
    });
  })));
};

Object.defineProperty(exports, 'createEditorStateFromRaw', {
  enumerable: true,
  get: function () {
    return draftjsConductor.createEditorStateFromRaw;
  }
});
Object.defineProperty(exports, 'serialiseEditorStateToRaw', {
  enumerable: true,
  get: function () {
    return draftjsConductor.serialiseEditorStateToRaw;
  }
});
exports.BLOCK_TYPE = BLOCK_TYPE;
exports.DraftUtils = DraftUtils;
exports.DraftailEditor = DraftailEditor;
exports.ENTITY_TYPE = ENTITY_TYPE;
exports.INLINE_STYLE = INLINE_STYLE;
exports.Icon = Icon;
exports.InlineToolbar = InlineToolbar;
exports.MetaToolbar = MetaToolbar;
exports.Toolbar = Toolbar;
exports.ToolbarButton = ToolbarButton;
